package com.qr.database.common;

import net.minidev.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.util.Map;

//@Controller
public class BaseController {

    @Autowired
    public RedisUtil redisUtil;

    public Long loginUserId;//登录用户id

    public Long companyId;//登录用户所属企业id

    public Short roleId;//用户角色id

    /**
     * 校验用户是否登录
     *
     * @return 是否登录
     */
    public boolean checkLogin(HttpServletRequest request) {
        String authorization = "";
        Cookie[] cookies = request.getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals("token")) {
                    authorization = cookie.getValue();
                }
            }
        }
        if (authorization == null || "".equals(authorization)) {
            return false;
        }
        Map<String, Object> payload = Jwt.validToken(authorization);
        String state = (String) payload.get("state");
        if (TokenState.VALID.getState().equals(state)) {
            JSONObject data = (JSONObject) payload.get("data");
            Number type = data.getAsNumber("type");
            String userId = data.getAsString("uid");
            String cid = data.getAsString("cid");
            String rid = data.getAsString("rid");
            String chk = data.getAsString("chk");//单点登录验证标识
            if (type == null || chk == null || "".equals(chk)) {
                return false;
            }
            Integer tokenType = type.intValue();
            if (!tokenType.equals(TokenType.WEB.value())) {
                return false;
            }
            String key = RedisType.TOKEN.value() + userId + tokenType;
            String refreshKey = RedisType.TOKEN_REFRESH.value() + userId + tokenType;
            //检查是否过期及单点登录
            if (!redisUtil.exists(key) || !chk.equals(redisUtil.get(key))) {
                return false;
            } else {
                loginUserId = Long.parseLong(userId);
                companyId = Long.parseLong(cid);
                roleId = Short.parseShort(rid);
                //refresh标识失效但token还未失效时自动延长有效时间
                if (redisUtil.get(refreshKey) == null) {
                    redisUtil.set(key, chk, 60L * 60 * 2);//时间和登录时一致
                    redisUtil.set(refreshKey, "", (60L * 60 * 2 - 60L * 30));
                }
            }
        } else {
            return false;
        }
        return true;
    }

    /**
     * 检验用户是否拥有权限
     *
     * @param functionId 功能id
     *                   1：二维码数据库  2：数据评价系统  3：电子签章系统  4：EWIS系统
     * @return 是否有该功能的权限
     */
    public boolean checkFunction(Integer functionId) {
        Map<Integer, Object> functions = (Map<Integer, Object>) redisUtil.get(RedisType.FUNCTION.value() + loginUserId);
        if (functions != null && functions.containsKey(functionId.toString())) {
            return true;
        }
        return false;
    }

}
