package com.qr.database.config.aop;

import com.qr.database.common.*;
import net.minidev.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

@Component
public class LoginInterceptor extends HandlerInterceptorAdapter {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    @Autowired
    public RedisUtil redisUtil;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
                             Object arg2) throws Exception {
        request.setAttribute("attrUserId","4");
        request.setAttribute("attrCompanyId","1");
        request.setAttribute("attrRoleId","1");
        return true;
//        request.setAttribute("attrUserId","2");
//        request.setAttribute("attrCompanyId","1");
//        request.setAttribute("attrRoleId","1");

//        boolean b = checkLogin(request);
//        if (b) {
//            boolean b1 = checkFunction(1, request.getAttribute("attrUserId"));
//            if (!b1) {
//                request.setAttribute("isAudit", 2);
//                request.getRequestDispatcher("/").forward(request,response);
//            }
//            return b1;
//        }
//        request.setAttribute("isAudit", 1);
//        request.getRequestDispatcher("/").forward(request,response);
//        return false;
    }

    /**
     * 校验用户是否登录
     *
     * @return 是否登录
     */
    public boolean checkLogin(HttpServletRequest request) {
        String authorization = "";
        Cookie[] cookies = request.getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals("token")) {
                    authorization = cookie.getValue();
                }
            }
        }
        if (authorization == null || "".equals(authorization)) {
            return false;
        }
        Map<String, Object> payload = Jwt.validToken(authorization);
        String state = (String) payload.get("state");
        if (TokenState.VALID.getState().equals(state)) {
            JSONObject data = (JSONObject) payload.get("data");
            Number type = data.getAsNumber("type");
            String userId = data.getAsString("uid");
            String cid = data.getAsString("cid");
            String rid = data.getAsString("rid");
            String chk = data.getAsString("chk");//单点登录验证标识
            if (type == null || chk == null || "".equals(chk)) {
                return false;
            }
            Integer tokenType = type.intValue();
            if (!tokenType.equals(TokenType.WEB.value())) {
                return false;
            }
            String key = RedisType.TOKEN.value() + userId + tokenType;
            String refreshKey = RedisType.TOKEN_REFRESH.value() + userId + tokenType;
            //检查是否过期及单点登录
            if (!redisUtil.exists(key) || !chk.equals(redisUtil.get(key))) {
                return false;
            } else {
                request.setAttribute("attrUserId",Long.parseLong(userId));
                request.setAttribute("attrCompanyId",Long.parseLong(cid));
                request.setAttribute("attrRoleId",Short.parseShort(rid));
//                loginUserId = Long.parseLong(userId);
//                companyId = Long.parseLong(cid);
//                roleId = Short.parseShort(rid);
                //refresh标识失效但token还未失效时自动延长有效时间
                if (redisUtil.get(refreshKey) == null) {
                    redisUtil.set(key, chk, 60L * 60 * 2);//时间和登录时一致
                    redisUtil.set(refreshKey, "", (60L * 60 * 2 - 60L * 30));
                }
            }
        } else {
            return false;
        }
        return true;
    }

    /**
     * 检验用户是否拥有权限
     *
     * @param functionId 功能id
     *                   1：二维码数据库  2：数据评价系统  3：电子签章系统  4：EWIS系统
     * @return 是否有该功能的权限
     */
    public boolean checkFunction(Integer functionId, Object loginUserId) {
        Map<Integer, Object> functions = (Map<Integer, Object>) redisUtil.get(RedisType.FUNCTION.value() + loginUserId);
        if (functions != null && functions.containsKey(functionId.toString())) {
            return true;
        }
        return false;
    }

}
