package com.qr.database.controller;

import cn.afterturn.easypoi.entity.ImageEntity;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qr.database.config.properties.GlobalProperties;
import com.qr.database.model.*;
import com.qr.database.service.*;
import com.qr.database.tips.ReturnResult;
import com.qr.database.tips.factory.AppPageFactory;
import com.qr.database.utils.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static oracle.net.aso.C01.o;

@Controller
@RequestMapping("/qr/shenqian")
public class ShenqianController {

    @Autowired
    private IShenqianService shenqianService;
    @Autowired
    private IProductService productService;
    @Autowired
    private IFlowNodeService flowNodeService;
    @Autowired
    private IShenqianLogService shenqianLogService;
    @Autowired
    private IFlowService flowService;
    @Autowired
    private ExportExcelUtil exportExcelUtil;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @RequestMapping("/start")
    @ResponseBody
    public ReturnResult start (@RequestParam("batchNos") List<String> batchNos, String note, String flowId,
                               @RequestAttribute("attrUserId") Long userId,@RequestAttribute("attrCompanyId") Long companyId) {
        String userName = shenqianService.getUserNameById(userId, null);
        int ok = 0;
        int fail = 0;
        Iterator<String> iterator = batchNos.iterator();
        while (iterator.hasNext()) {
            String batchNo = iterator.next();
            boolean start = shenqianService.start(batchNo, note, flowId, userId, companyId, userName);
            if (start) {
                ok ++;
                iterator.remove();
            } else {
                fail ++;
            }
        }
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("ok", ok);
        stringObjectHashMap.put("fall", fail);
        stringObjectHashMap.put("batchNos", batchNos);
        return ReturnResult.success(stringObjectHashMap);
    }

    @RequestMapping("/restart")
    @ResponseBody
    public ReturnResult restart(String note, String flowId, String id,
                               @RequestAttribute("attrUserId") Long userId,@RequestAttribute("attrCompanyId") Long companyId) {
        String userName = shenqianService.getUserNameById(userId, null);
        return shenqianService.restart(id, flowId, note, userId, userName);
    }

    @RequestMapping("/restartPage")
    public ModelAndView restartPage(String id,ModelAndView modelAndView,
                                @RequestAttribute("attrUserId") Long userId,@RequestAttribute("attrCompanyId") Long companyId) {
        List<ShenqianLog> list = shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).orderByAsc("ORDER_NUMBER"));
        Map<String, Object> flowSelect = flowService.getFlowSelect(null);
        Shenqian byId = shenqianService.getById(id);
        Product byId1 = productService.getById(byId.getProductId());
        modelAndView.addObject("shenqianLog", list);
        modelAndView.addObject("param", byId1.getBatchNo());
        modelAndView.addObject("id", id);
        modelAndView.addObject("flowSelect", flowSelect);
        modelAndView.setViewName("productReStartShenQian");
        return modelAndView;
    }

    @RequestMapping("/list")
    public ModelAndView list(ModelAndView modelAndView, BindingResult result, Shenqian shenqian, Integer all,
                             @RequestAttribute("attrUserId") Long userId, @RequestAttribute("attrCompanyId") Long companyId, @RequestAttribute("attrRoleId") Long roleId) {
        int count = shenqianService.count(new QueryWrapper<Shenqian>().eq("APPROVAL_USER_ID", userId).eq("FLAG", 1));
        if (all== null || all==1) {
            shenqian.setUserId(userId);
            modelAndView.addObject("count", shenqianService.count(new QueryWrapper<Shenqian>().eq("USER_ID", userId).eq("FLAG", 1)));
            modelAndView.addObject("waitCount", count);
        } else if (all == 2) {
            shenqian.setApprovalUserId(userId);
            modelAndView.addObject("count", count);
        } else {
            modelAndView.addObject("count", shenqianService.count(new QueryWrapper<Shenqian>().eq("FLAG", 1)));
            modelAndView.addObject("waitCount", count);
        }
        Page<Shenqian> page = new AppPageFactory<Shenqian>().defaultPage();
        page.setRecords((List) shenqianService.listShenqian(page, shenqian));
        modelAndView.addObject("page", page);
        modelAndView.addObject("pages", page.getPages());
        modelAndView.addObject("param", shenqian);
        modelAndView.addObject("roleId", roleId);
        List<String> node_name = (List)flowNodeService.listObjs(new QueryWrapper<FlowNode>().select("NODE_NAME"));
        modelAndView.addObject("nodeNames", new HashSet<String>(node_name));
        modelAndView.setViewName("shenqianList");
        return modelAndView;
    }

    @RequestMapping("/awaitList")
    public ModelAndView awaitList(ModelAndView modelAndView, BindingResult result, Shenqian shenqian,
                             @RequestAttribute("attrUserId") Long userId, @RequestAttribute("attrCompanyId") Long companyId, @RequestAttribute("attrRoleId") Long roleId) {
        int count = shenqianService.count(new QueryWrapper<Shenqian>().eq("APPROVAL_USER_ID", userId).eq("FLAG", 1));
        shenqian.setApprovalUserId(userId);
        modelAndView.addObject("count", count);

        Page<Shenqian> page = new AppPageFactory<Shenqian>().defaultPage();
        page.setRecords((List) shenqianService.listShenqian(page, shenqian));
        modelAndView.addObject("page", page);
        modelAndView.addObject("pages", page.getPages());
        modelAndView.addObject("param", shenqian);
        modelAndView.addObject("roleId", roleId);
        List<String> node_name = (List)flowNodeService.listObjs(new QueryWrapper<FlowNode>().select("NODE_NAME"));
        modelAndView.addObject("nodeNames", new HashSet<String>(node_name));
        modelAndView.setViewName("shenqianAwaitList");
        return modelAndView;
    }

    @RequestMapping("/allList")
    public ModelAndView allList(ModelAndView modelAndView, BindingResult result, Shenqian shenqian,
                                  @RequestAttribute("attrUserId") Long userId, @RequestAttribute("attrCompanyId") Long companyId) {
        int count = shenqianService.count(new QueryWrapper<Shenqian>().eq("APPROVAL_USER_ID", userId).eq("FLAG", 1));
        modelAndView.addObject("count", shenqianService.count(new QueryWrapper<Shenqian>().eq("FLAG", 1)));
        modelAndView.addObject("waitCount", count);
        shenqian.setCompanyId(companyId);
        Page<Shenqian> page = new AppPageFactory<Shenqian>().defaultPage();
        page.setRecords((List) shenqianService.listShenqian(page, shenqian));
        modelAndView.addObject("page", page);
        modelAndView.addObject("pages", page.getPages());
        modelAndView.addObject("param", shenqian);
        List<String> node_name = (List)flowNodeService.listObjs(new QueryWrapper<FlowNode>().select("NODE_NAME"));
        modelAndView.addObject("nodeNames", new HashSet<String>(node_name));
        modelAndView.setViewName("shenqianAllList");
        return modelAndView;
    }

    @RequestMapping("/detail/{id}")
    public ModelAndView detail(@PathVariable String id, ModelAndView modelAndView) {
        Shenqian byId = shenqianService.getById(id);
        if (byId != null) {
            List<ShenqianLog> list = shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).orderByAsc("ORDER_NUMBER"));
            byId.setShenqianLogList(list);
            Product detail = productService.detail(String.valueOf(byId.getProductId()));
            modelAndView.addObject("product", detail);
        }
        modelAndView.addObject("shenqian", byId);
        modelAndView.setViewName("shenqianDetail");
        return modelAndView;
    }

    @RequestMapping("/awaitDetail/{id}")
    public ModelAndView awaitDetail(@PathVariable String id, @RequestAttribute("attrUserId") Long userId, @RequestAttribute("attrRoleId") Long roleId, ModelAndView modelAndView) {
        Shenqian byId = shenqianService.getById(id);
        if (byId != null) {
            List<ShenqianLog> list = shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).orderByAsc("ORDER_NUMBER"));
            byId.setShenqianLogList(list);
            Product detail = productService.detail(String.valueOf(byId.getProductId()));
            modelAndView.addObject("product", detail);
        }
        Shenqian last = shenqianService.last(id, null, userId);
        Shenqian next = shenqianService.next(id, null, userId);

        modelAndView.addObject("last", last);
        modelAndView.addObject("next", next);
        modelAndView.addObject("shenqian", byId);
        modelAndView.addObject("userId", userId);
        modelAndView.setViewName("shenqianAwaitDetail");
        return modelAndView;
    }

    @RequestMapping("/audit")
    @ResponseBody
    public ReturnResult audit(String id, String note, boolean audit,@RequestAttribute("attrUserId") Long userId) {
        Shenqian byId = shenqianService.getById(id);
        if (byId != null) {
            Date date = new Date();
            Shenqian shenqian = new Shenqian();
            ShenqianLog shenqianLog = new ShenqianLog();
            shenqianLog.setNote(note);
            shenqianLog.setCreateTime(date);
            Long step = byId.getStep();
            Map<String, Object> map = null;
            if (audit) {
                ShenqianLog one = shenqianLogService.getOne(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).eq("ORDER_NUMBER", step + 1));
                if (one !=null) {
                    shenqian.setStep(step+1);
                    shenqian.setApprovalUserId(one.getApprovalUserId());
                } else {
                    shenqian.setStatus(4);
                    shenqian.setDocPath("shenqianDoc"+File.separator+byId.getOrderId()+".docx");
                    map = new HashMap<>();
                    map.put("fileName", "shenqianDoc"+File.separator+byId.getOrderId()+".docx");
                    Standard standard = productService.shenqianDoc(byId.getProductId());
                    map.put("shipmentsAmount", standard.getShipmentsAmount());
                    map.put("name", standard.getName());
                    map.put("batchMarking", standard.getBatchMarking());
                    map.put("unit", standard.getUnit());
                    map.put("year", DateUtil.getYear(date));
                    map.put("month", DateUtil.getMonth(date));
                    map.put("day", DateUtil.getDay(date));
                    List<String> strings = shenqianLogService.shenqianDocApprovalUser(byId.getId());
                    int size = strings.size();
                    for (int i=0; i<5;i++) {
                        if (size>i) {
                            ImageEntity image = new ImageEntity();
                            image.setHeight(100);
                            image.setWidth(100);
                            image.setUrl(strings.get(i));
                            image.setType(ImageEntity.URL);
                            map.put("img"+i, image);
                        } else {
                            map.put("img"+i, " ");
                        }
                    }
                }
                boolean update = shenqian.update(new QueryWrapper<Shenqian>().eq("ID", id).eq("STATUS", 1).eq("APPROVAL_USER_ID", userId));
                if (update) {
                    shenqianLog.setStatus("通过");
                    shenqianLog.update(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).eq("ORDER_NUMBER", step));
                    if (map != null) {
                        DocumentUtil.export(map, "shenqian.docx");
                    }
                    return ReturnResult.success();
                }
                return ReturnResult.fail("审签单状态不正确");
            } else {
                shenqian.setStatus(2);
                boolean update = shenqian.update(new QueryWrapper<Shenqian>().eq("ID", id).eq("STATUS", 1).eq("APPROVAL_USER_ID", userId));
                if (update) {
                    shenqianLog.setStatus("驳回");
                    shenqianLog.update(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).eq("ORDER_NUMBER", step));
                    return ReturnResult.success();
                }
                return ReturnResult.fail("审签单状态不正确");
            }
        }
        return ReturnResult.fail("审签单不存在");
    }

    @RequestMapping("/del")
    @ResponseBody
    public ReturnResult del(String id, @RequestAttribute("attrUserId") Long userId) {
        Shenqian shenqian = new Shenqian();
        shenqian.setFlag(2);
        boolean update = shenqian.update(new UpdateWrapper<Shenqian>().eq("ID", id).eq("STATUS", 3));
        if (update) {
            return ReturnResult.success();
        }
        return ReturnResult.fail("审签单状态不正确");
    }

    @RequestMapping("/preview/{id}/{docType}")
    @ResponseBody
    public ReturnResult preview(@PathVariable("id") String id, @PathVariable("docType") Integer docType, @RequestAttribute("attrUserId") Long userId) {
        Shenqian byId = shenqianService.getById(id);
        if (byId !=null) {
            String uuid = CommonUtil.getUUID();
            ReturnResult returnResult = new ReturnResult(300, "1", uuid);
            redisTemplate.opsForValue().set(uuid, returnResult, 10, TimeUnit.MINUTES);
            Product byId1 = productService.getById(byId.getProductId());
            Shenqian last = shenqianService.last(id, null, userId);
            Shenqian next = shenqianService.next(id, null, userId);
            HashMap<String, Object> stringLongHashMap = new HashMap<>();
            stringLongHashMap.put("status", byId.getStatus());
            stringLongHashMap.put("id", byId.getId());
            stringLongHashMap.put("userId", userId);
            stringLongHashMap.put("approvalUserId", byId.getApprovalUserId());
            stringLongHashMap.put("last", last==null?null:last.getId());
            stringLongHashMap.put("next", next==null?null:next.getId());
            ShenQianPreviewUtil previewUtil = new ShenQianPreviewUtil(uuid, stringLongHashMap, byId1, docType);
            previewUtil.start();
            return returnResult;
        }
        return ReturnResult.fail("数据不存在");
    }

    @RequestMapping("/previewFinish/{id}")
    public ModelAndView previewFinish(@PathVariable("id") String id, ModelAndView modelAndView) {
        LinkedHashMap map = (LinkedHashMap)redisTemplate.opsForValue().get(id);
        Object shenqian = map.remove("shenqian");
        modelAndView.addObject("param", map);
        modelAndView.addObject("shenqian", shenqian);
        modelAndView.setViewName("shenqianDocPreview");
        new Thread(){
            @Override
            public void run() {
                redisTemplate.delete(id);
            }
        }.start();
        return modelAndView;
    }

    @RequestMapping(value = "/exportLog/{id}")
    public String export(ModelMap modelMap, @PathVariable String id) {
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("maplist", shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).orderByAsc("ORDER_NUMBER")));
        return exportExcelUtil.export(modelMap,"shenqianLog.xlsx","审批记录"+ DateUtil.getAllTime(), stringObjectHashMap);
    }

    @RequestMapping(value = "/finishExport")
    public void finishExport(@RequestParam("ids") List<String> ids, HttpServletResponse response) throws IOException {
        Collection<Shenqian> shenqians = shenqianService.listByIds(ids);
        List<Map<String, Object>> parms1 = new LinkedList<>();
        for (Shenqian s:shenqians) {
            String docPath = s.getDocPath();
            if (StringUtils.isNotBlank(docPath)) {
                parms1.add(ZipUtil.getZipMap(docPath));
            }
            HashMap<String, Object> stringObjectHashMap = new HashMap<>();
            stringObjectHashMap.put("maplist", shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", s.getId()).orderByAsc("ORDER_NUMBER")));
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ExportExcelUtil.export(stringObjectHashMap, "shenqianLog.xlsx", baos);
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            parms1.add(stringObjectHashMap1);
            stringObjectHashMap1.put("fileName", s.getOrderId()+".xlsx");
            stringObjectHashMap1.put("data", baos.toByteArray());
            baos.close();
        }
        response.setContentType("application/octet-stream;charset=UTF-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + new String("审签记录.zip".getBytes(), "iso-8859-1"));
        ZipUtil.batchAllToZIP(parms1, response.getOutputStream());
    }

    @RequestMapping(value = "/allExport/{id}")
    public void allExport(@PathVariable String id, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> parms1 = new LinkedList<>();
        Shenqian byId = shenqianService.getById(id);
        if (byId != null) {
            String docPath = byId.getDocPath();
            if (StringUtils.isNotBlank(docPath)) {
                parms1.add(ZipUtil.getZipMap(docPath));
            }
            HashMap<String, Object> stringObjectHashMap = new HashMap<>();
            stringObjectHashMap.put("maplist", shenqianLogService.list(new QueryWrapper<ShenqianLog>().eq("SHENQIAN_ID", byId.getId()).orderByAsc("ORDER_NUMBER")));
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ExportExcelUtil.export(stringObjectHashMap, "shenqianLog.xlsx", baos);
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            parms1.add(stringObjectHashMap1);
            stringObjectHashMap1.put("fileName", byId.getOrderId()+".xlsx");
            stringObjectHashMap1.put("data", baos.toByteArray());
            baos.close();
            Product product = productService.getById(byId.getProductId());
            String surveyReport = product.getSurveyReport();
            if (StringUtils.isNotBlank(surveyReport)) {
                parms1.add(ZipUtil.getZipMap(surveyReport));
            }
            String outgoingReport = product.getOutgoingReport();
            if (StringUtils.isNotBlank(outgoingReport)) {
                parms1.add(ZipUtil.getZipMap(outgoingReport));
            }
            String intoPerReport = product.getIntoPerReport();
            if (StringUtils.isNotBlank(intoPerReport)) {
                parms1.add(ZipUtil.getZipMap(intoPerReport));
            }
            String intoSizeReport = product.getIntoSizeReport();
            if (StringUtils.isNotBlank(intoSizeReport)) {
                parms1.add(ZipUtil.getZipMap(intoSizeReport));
            }
            String thirdpartyPerReport = product.getThirdpartyPerReport();
            if (StringUtils.isNotBlank(thirdpartyPerReport)) {
                parms1.add(ZipUtil.getZipMap(thirdpartyPerReport));
            }
            String thirdpartySizeReport = product.getThirdpartySizeReport();
            if (StringUtils.isNotBlank(thirdpartySizeReport)) {
                parms1.add(ZipUtil.getZipMap(thirdpartySizeReport));
            }
            String certification = product.getCertification();
            if (StringUtils.isNotBlank(certification)) {
                parms1.add(ZipUtil.getZipMap(certification));
            }
            String subCertification = product.getSubCertification();
            if (StringUtils.isNotBlank(subCertification)) {
                HashMap<String, Object> subCertificationMap = new HashMap<>();
                File file = new File(GlobalProperties.getFileUploadPath() + subCertification);
                subCertificationMap.put("fileName", "子合格证-"+file.getName());
                subCertificationMap.put("file", new FileInputStream(file));
                parms1.add(subCertificationMap);
            }
        }
        response.setContentType("application/octet-stream;charset=UTF-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + new String("审签单文档.zip".getBytes(), "iso-8859-1"));
        ZipUtil.batchAllToZIP(parms1, response.getOutputStream());
    }

    @Scheduled(cron = "0 */5 * * * ?")
    public void report() {
        List<Shenqian> status = shenqianService.list(new QueryWrapper<Shenqian>().eq("STATUS", 1));
        for (Shenqian s:status) {
            String userName = shenqianService.getUserNameById(s.getApprovalUserId(), 0);
            if (userName == null) {
                shenqianService.errUpdate(s);
            }
        }
    }
}
