package com.qr.database.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qr.database.config.GlobalConstant;
import com.qr.database.config.properties.GlobalProperties;
import com.qr.database.model.Product;
import com.qr.database.model.Standard;
import com.qr.database.service.IProductService;
import com.qr.database.service.IStandardService;
import com.qr.database.tips.ReturnResult;
import com.qr.database.tips.factory.AppPageFactory;
import com.qr.database.utils.*;
import com.qr.database.utils.qr.ImgQrTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Controller
@RequestMapping("/qr/standard")
public class StandardController {

    private static Logger log = LoggerFactory.getLogger(StandardController.class);
    @Autowired
    private IStandardService standardService;
    @Autowired
    private IProductService productService;
    @Autowired
    private ExportExcelUtil exportExcelUtil;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @RequestMapping("/list")
    public ModelAndView list(ModelAndView modelAndView, Standard standard, @RequestAttribute("attrCompanyId") Long companyId) {
        Integer isStandard = standard.getIsStandard();
        standard.setCompanyId(companyId);
        if (isStandard == null || isStandard != 2) {
            Page<Product> page = new AppPageFactory<Product>().defaultPage();
            page.setRecords((List) standardService.listProduct(page, standard));
            modelAndView.addObject("count", productService.count());
            modelAndView.addObject("page", page);
            modelAndView.addObject("pages", page.getPages());
        } else {
            Page<Standard> page = new AppPageFactory<Standard>().defaultPage();
            page.setRecords((List) standardService.list(page, standard));
            modelAndView.addObject("count", standardService.count());
            modelAndView.addObject("page", page);
            modelAndView.addObject("pages", page.getPages());
        }
        modelAndView.addObject("isStandard", standard.getIsStandard());
        modelAndView.addObject("param", standard);
        modelAndView.setViewName("standardList");
        return modelAndView;
    }

    @RequestMapping("/imports")
    @ResponseBody
    public ReturnResult imports(@NotNull(message = "上传文件不能为空") @RequestPart(value = "file", required = false) MultipartFile picture, @RequestAttribute("attrCompanyId") Long companyId) throws Exception {
        List<Standard> imports = ExcelUtil.imports(picture.getInputStream(), Standard.class);
        Iterator<Standard> iterator = imports.iterator();
        Date date = new Date();
        int ok = 0;
        int fall = 0;
        LinkedList<Standard> standards = new LinkedList<>();
        LinkedList<String> okList = new LinkedList<>();
        String uuid = CommonUtil.getUUID();
        HashMap<String, Object> stringIntegerHashMap = new HashMap<>();
        while (iterator.hasNext()) {
            Standard next = iterator.next();
            next.setCreateTime(date);
            next.setCompanyId(companyId);
            boolean add = standardService.add(next);
            if (add) {
                ok++;
                okList.add(next.getGroupItemCode());
            } else {
                fall++;
                standards.add(next);
            }
        }
        if (ok >= 1) {
            redisTemplate.opsForValue().set(GlobalConstant.OK_NEWS + uuid, okList, 8, TimeUnit.HOURS);
        }
        if (fall >= 1) {
            redisTemplate.opsForValue().set(GlobalConstant.FALL_NEWS + uuid, standards, 8, TimeUnit.HOURS);
        }
        stringIntegerHashMap.put("ok", ok);
        stringIntegerHashMap.put("fall", fall);
        stringIntegerHashMap.put("id", uuid);
        stringIntegerHashMap.put("flag", "standard");
        return ReturnResult.success(stringIntegerHashMap);
    }

    @RequestMapping(value = "/fall/{id}")
    public String export(@PathVariable("id") String id, ModelMap modelMap) {
        Object o = redisTemplate.opsForValue().get(GlobalConstant.FALL_NEWS + id);
        LinkedList<Standard> list = null;
        if (o != null) {
            list = (LinkedList) o;
        } else {
            list = new LinkedList<>();
        }
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("maplist", list);
        return exportExcelUtil.export(modelMap, "standard.xlsx", "标准导入出错数据", stringObjectHashMap);
    }

    @RequestMapping(value = "/ok/{id}")
    public void export(@PathVariable("id") String id, HttpServletResponse response) throws IOException {
        Object o = redisTemplate.opsForValue().get(GlobalConstant.OK_NEWS + id);
        List<Map<String, Object>> parms = new LinkedList<Map<String, Object>>();
        if (o != null) {
            List<Map<String, Object>> id1 = standardService.listMaps(new QueryWrapper<Standard>().select("GROUP_ITEM_CODE \"groupItemCode\"", "QR_DATA as \"qrData\"").in("GROUP_ITEM_CODE", (LinkedList) o));
            for (Map<String, Object> map : id1) {
                HashMap<String, Object> stringObjectHashMap = new HashMap<>();
                stringObjectHashMap.put("fileName", map.get("groupItemCode") + ".png");
                stringObjectHashMap.put("data", ImgQrTool.Base64ToImage(String.valueOf(map.get("qrData"))));
                parms.add(stringObjectHashMap);
            }
        }
        response.setContentType("application/octet-stream;charset=UTF-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + new String("标准二维码.zip".getBytes(), "iso-8859-1"));
        ZipUtil.batchDataToZIP(parms, response.getOutputStream());
    }

    @RequestMapping("/importDoc")
    @ResponseBody
    public ReturnResult importDoc(@NotNull(message = "上传文件不能为空") @RequestPart(value = "file", required = false) MultipartFile picture) throws Exception {
        String fileSavePath = GlobalProperties.getFileUploadPath() + "standardDoc" + File.separator;
        String upload = FileUtil.upload(picture, fileSavePath, picture.getOriginalFilename());
        if (upload == null) {
            return ReturnResult.fail(480, "文件上传失败");
        }
        return ReturnResult.success();
    }

    @RequestMapping("/detail/{id}")
    public ModelAndView detail(@PathVariable("id") String id, ModelAndView modelAndView) {
        Standard byId = standardService.getById(id);
        modelAndView.addObject("standard", byId==null?new Standard():byId);
        modelAndView.setViewName("standardDetail");
        return modelAndView;
    }

    @RequestMapping("/ewmDetail")
    public ModelAndView ewmDetail(String qrId, ModelAndView modelAndView) {
        String[] split = qrId.split(GlobalConstant.GS);
        try {
            if (split.length==4) {
                Product product = productService.ewmDetail(split[1].substring(1), split[2].substring(1), split[3].substring(1, split[3].length() - 2));
                modelAndView.addObject("product", product);
                modelAndView.setViewName("productDetail");
                return modelAndView;
            } else if (split.length==2) {
                Standard byId = standardService.getOne(new QueryWrapper<Standard>().eq("GROUP_ITEM_CODE", split[1].substring(1, split[1].length() - 2)));
                modelAndView.addObject("standard", byId==null?new Standard():byId);
                modelAndView.setViewName("standardDetail");
                return modelAndView;
            }
        } catch (RuntimeException e) {
            log.warn(qrId+"-"+e.getMessage());
        }
        modelAndView.addObject("standard", new Standard());
        modelAndView.setViewName("standardDetail");
        return modelAndView;
    }
}
