package com.qr.database.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qr.database.config.GlobalConstant;
import com.qr.database.dao.ProductMapper;
import com.qr.database.dao.StandardMapper;
import com.qr.database.model.Product;
import com.qr.database.model.ProductExport;
import com.qr.database.model.Standard;
import com.qr.database.service.IProductService;
import com.qr.database.utils.FileUtil;
import com.qr.database.utils.qr.ImgQrTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.plaf.TableHeaderUI;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author stylefeng123
 * @since 2019-07-02
 */
@Service
public class ProductServiceImpl extends ServiceImpl<ProductMapper, Product> implements IProductService {

    private static String prefix = "D>"+ GlobalConstant.RS+"21"+GlobalConstant.GS+"M";
    private static String suffix = GlobalConstant.RS+GlobalConstant.EOT;
    private static Logger log = LoggerFactory.getLogger(ProductServiceImpl.class);

    @Autowired
    private StandardMapper standardMapper;

    @Override
    public boolean add(Product next, Map<String, String> map) {
        try {
            String batchNo = next.getBatchNo();
            if (StringUtils.isBlank(batchNo)) {
                return false;
            }
            Pattern compile = Pattern.compile(GlobalConstant.UNIT_FLAG);
            Matcher matcher = compile.matcher(batchNo);
            if (!matcher.find()) {
                return false;
            }
            String group = matcher.group();
            next.setFlag(group);
            next.setNo(batchNo.replaceFirst(group,""));
            String s = map.get(group);
            if (s == null) {
                return false;
            }
            Standard batch_marking = standardMapper.selectOne(new QueryWrapper<Standard>().eq("BATCH_MARKING", next.getBatchMarking()));
            if (batch_marking == null) {
                return false;
            }
            next.setQrData(ImgQrTool.createSimpleQr(prefix+ s +GlobalConstant.GS+"W"+batch_marking.getGroupItemCode()+GlobalConstant.GS+"S"+next.getNo()+suffix, 150,150));
            return next.insert();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return false;
        }
    }

    @Override
    @Transactional
    public boolean del(Product product, List<Integer> delDoc, Integer delType) {
        if (delType==2) {
            Product product1 = new Product();
            product1.setStatus(2);
            product1.setId(product.getId());
            if (product1.updateById()) {
                delFile(product, delDoc);
                shenqianErrUpdate(product.getBatchNo());
                return true;
            }
            return false;
        } else if (delType==1) {
            if (delDoc == null || delDoc.size()==0) {
                return true;
            }
            Product product1 = new Product();
            product1.setId(product.getId());
            boolean isUpdate = false;
            for (Integer f:delDoc) {
                if (f==1) {
                    product1.setSurveyReport("");
                    isUpdate = true;
                } else if (f==2) {
                    product1.setOutgoingReport("");
                    isUpdate = true;
                } else if (f==3) {
                    product1.setThirdpartySizeReport("");
                    isUpdate = true;
                } else if (f==4) {
                    product1.setThirdpartyPerReport("");
                    isUpdate = true;
                } else if (f==5) {
                    product1.setIntoSizeReport("");
                    isUpdate = true;
                } else if (f==6) {
                    product1.setIntoPerReport("");
                    isUpdate = true;
                } else if (f==7) {
                    product1.setCertification("");
                    isUpdate = true;
                } else if (f==8) {
                    product1.setSubCertification("");
                    isUpdate = true;
                }
            }
            if (isUpdate && product1.updateById()) {
                delFile(product, delDoc);
                shenqianErrUpdate(product.getBatchNo());
                return true;
            }
        }
        return false;
    }

    @Override
    public List<ProductExport> selectListByBatchNo(List<String> batchNo) {
        return this.baseMapper.selectListByBatchNo(batchNo);
    }

    @Override
    public Product detail(String id) {
        return this.baseMapper.detail(id);
    }

    @Override
    public Standard shenqianDoc(Long productId) {
        return this.baseMapper.shenqianDoc(productId);
    }

    @Override
    public void shenqianErrUpdate(String batchNo) {
        this.baseMapper.shenqianErrUpdate(batchNo);
    }

    public void delFile(Product product, List<Integer> delDoc) {
        for (Integer f:delDoc) {
            if (f==1) {
                FileUtil.deleteFile(product.getSurveyReport());
            } else if (f==2) {
                FileUtil.deleteFile(product.getOutgoingReport());
            } else if (f==3) {
                FileUtil.deleteFile(product.getThirdpartySizeReport());
            } else if (f==4) {
                FileUtil.deleteFile(product.getThirdpartyPerReport());
            } else if (f==5) {
                FileUtil.deleteFile(product.getIntoSizeReport());
            } else if (f==6) {
                FileUtil.deleteFile(product.getIntoPerReport());
            } else if (f==7) {
                FileUtil.deleteFile(product.getCertification());
            } else if (f==8) {
                FileUtil.deleteFile(product.getSubCertification());
            }
        }
    }

    @Override
    public Product ewmDetail(String unitCode, String groupItemCode, String no) {
        return this.baseMapper.ewmDetail(unitCode, groupItemCode, no);
    }

}
