package com.qr.database.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qr.database.dao.ProductMapper;
import com.qr.database.dao.ShenqianMapper;
import com.qr.database.model.Flow;
import com.qr.database.model.Product;
import com.qr.database.model.Shenqian;
import com.qr.database.model.ShenqianLog;
import com.qr.database.service.IFlowService;
import com.qr.database.service.IShenqianLogService;
import com.qr.database.service.IShenqianService;
import com.qr.database.tips.ReturnResult;
import com.qr.database.utils.CommonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>
 * 审签 服务实现类
 * </p>
 *
 * @author stylefeng123
 * @since 2019-07-08
 */
@Service
public class ShenqianServiceImpl extends ServiceImpl<ShenqianMapper, Shenqian> implements IShenqianService {

    @Autowired
    private ProductMapper productMapper;
    @Autowired
    private IFlowService flowService;
    @Autowired
    private IShenqianLogService shenqianLogService;

    @Override
    @Transactional
    public boolean start(String batchNo, String note, String flowId, Long userId, Long companyId, String userName) {
        try {
            Product product = productMapper.selectOne(new QueryWrapper<Product>().eq("BATCH_NO", batchNo));
            if (product == null) {
                return false;
            }
            List<Flow> flow = flowService.getFlow(Long.valueOf(flowId));
            if (flow.size()<=0) {
                return false;
            }
            Shenqian shenqian = new Shenqian();
            shenqian.setProductId(product.getId());
            shenqian.setFlowId(Long.valueOf(flowId));
            shenqian.setOrderId(CommonUtil.getShenqianId());
            shenqian.setCreateTime(new Date());
            shenqian.setUserId(userId);
            shenqian.setCompanyId(companyId);
            shenqian.setStatus(1);
            shenqian.setStep(1L);
            shenqian.setFlag(1);
            LinkedList<ShenqianLog> shenqianLogs = new LinkedList<>();
            for (long i = 1; i <= flow.size(); i++) {
                Flow flow1 = flow.get((int) (i - 1));
                ShenqianLog shenqianLog = new ShenqianLog();
                shenqianLog.setOrderNumber(i);
                shenqianLog.setFlowId(Long.valueOf(flowId));
                shenqianLog.setNodeName(flow1.getNodeName());
                shenqianLog.setApprovalType(flow1.getApprovalType());
                Long userId1 = flow1.getUserId();
                if (userId1 == null) {
                    return false;
                }
                if (i==1) {
                    shenqian.setApprovalUserId(userId1);
                }
                shenqianLog.setApprovalUserId(userId1);
                shenqianLog.setApprovalUserName(flow1.getNickname());
                shenqianLogs.add(shenqianLog);
            }
            if (shenqian.insert()) {
                for (ShenqianLog s: shenqianLogs) {
                    s.setShenqianId(shenqian.getId());
                }
                ShenqianLog shenqianLog = new ShenqianLog();
                shenqianLog.setOrderNumber(0l);
                shenqianLog.setShenqianId(shenqian.getId());
                shenqianLog.setFlowId(Long.valueOf(flowId));
                shenqianLog.setNodeName("发起");
                shenqianLog.setApprovalUserId(userId);
                shenqianLog.setApprovalUserName(userName);
                shenqianLog.setCreateTime(new Date());
                shenqianLog.setStatus("发起审批");
                shenqianLog.setNote(note);
                shenqianLogs.addFirst(shenqianLog);
                boolean b = shenqianLogService.saveBatch(shenqianLogs);
                if (b) {
                    return true;
                }
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;
    }


    @Override
    public String getUserNameById(Long userId, Integer status) {
        return this.baseMapper.getUserNameById(userId, status);
    }

    @Override
    public List<Shenqian> listShenqian(Page<Shenqian> page, Shenqian shenqian) {
        return this.baseMapper.listShenqian(page, shenqian);
    }

    @Override
    @Transactional
    public ReturnResult restart(String id, String flowId, String note, Long userId, String userName) {
        Shenqian byId = this.getById(id);
        if (byId != null) {
            List<Flow> flow = flowService.getFlow(Long.valueOf(flowId));
            if (flow.size()<=0) {
                return ReturnResult.fail("流程不存在");
            }
            LinkedList<ShenqianLog> shenqianLogs = new LinkedList<>();
            Iterator<Flow> iterator = flow.iterator();
            long i = byId.getStep()+1;
            Shenqian shenqian = new Shenqian();
            shenqian.setStatus(1);
            shenqian.setStep(i+1);
            ShenqianLog shenqianLog1 = new ShenqianLog();
            shenqianLog1.setOrderNumber(i);
            shenqianLog1.setShenqianId(byId.getId());
            shenqianLog1.setFlowId(Long.valueOf(flowId));
            shenqianLog1.setNodeName("重新发起");
            shenqianLog1.setApprovalUserId(userId);
            shenqianLog1.setApprovalUserName(userName);
            shenqianLog1.setCreateTime(new Date());
            shenqianLog1.setStatus("重新发起审批");
            shenqianLog1.setNote(note);
            shenqianLogs.add(shenqianLog1);
            while (iterator.hasNext()) {
                Flow next = iterator.next();
                if (i==byId.getStep()+1) {
                    shenqian.setApprovalUserId(next.getUserId());
                }
                i++;
                ShenqianLog shenqianLog = new ShenqianLog();
                shenqianLog.setOrderNumber(i);
                shenqianLog.setFlowId(Long.valueOf(flowId));
                shenqianLog.setNodeName(next.getNodeName());
                shenqianLog.setApprovalType(next.getApprovalType());
                Long userId1 = next.getUserId();
                if (userId1 == null) {
                    return ReturnResult.fail("流程中审批人不存在");
                }
                shenqianLog.setApprovalUserId(userId1);
                shenqianLog.setApprovalUserName(next.getNickname());
                shenqianLog.setShenqianId(byId.getId());
                shenqianLogs.add(shenqianLog);
            }
            boolean update = shenqian.update(new UpdateWrapper<Shenqian>().eq("ID", id).eq("FLAG",1).in("STATUS", 2, 3).eq("USER_ID", userId));
            if (update) {
                shenqianLogService.remove(new UpdateWrapper<ShenqianLog>().eq("SHENQIAN_ID", id).isNull("CREATE_TIME"));
                shenqianLogService.saveBatch(shenqianLogs);
                return ReturnResult.success();
            }
            return ReturnResult.fail("审签单状态不正确");
        }
        return ReturnResult.fail("审签单不存在");
    }

    @Override
    public Shenqian last(String id, Long userId, Long approvalUserId) {
        return this.baseMapper.last(id, userId, approvalUserId);
    }

    @Override
    public Shenqian next(String id, Long userId, Long approvalUserId) {
        return this.baseMapper.next(id, userId, approvalUserId);
    }

    @Override
    public void errUpdate(Shenqian s) {
        Shenqian shenqian = new Shenqian();
        shenqian.setStatus(3);
        boolean status1 = shenqian.update(new UpdateWrapper<Shenqian>().eq("STATUS", 1));
        if (status1) {
            ShenqianLog shenqianLog = new ShenqianLog();
            shenqianLog.setStatus("异常");
            shenqianLog.update(new UpdateWrapper<ShenqianLog>().eq("SHENQIAN_ID", s.getId()).eq("ORDER_NUMBER",s.getStep()).eq("APPROVAL_USER_ID", s.getApprovalUserId()));
        }
    }
}
