/**
 * Copyright (c) 2015-2016, Chill Zhuang 庄骞 (smallchill@163.com).
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.qr.database.utils;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class DateUtil {


    /**
     * 获取YYYY格式
     *
     * @return
     */
    public static String getYear() {
        return formatDate(new Date(), "yyyy");
    }

    /**
     * 获取YYYY格式
     *
     * @return
     */
    public static String getYear(Date date) {
        return formatDate(date, "yyyy");
    }

    /**
     * 获取YYYY-MM-DD格式
     *
     * @return
     */
    public static String getDay() {
        return formatDate(new Date(), "yyyy-MM-dd");
    }

    /**
     * 获取YYYY-MM-DD格式
     *
     * @return
     */
    public static String getDay(Date date) {
        return formatDate(date, "dd");
    }

    /**
     * 获取YYYYMMDD格式
     *
     * @return
     */
    public static String getDays() {
        return formatDate(new Date(), "yyyyMMdd");
    }

    /**
     * 获取yyyy-MMD格式
     *
     * @return
     */
    public static String getMonth(Date date) {
        return formatDate(date, "MM");
    }

    /**
     * 获取YYYYMMDD格式
     *
     * @return
     */
    public static String getDays(Date date) {
        return formatDate(date, "yyyyMMdd");
    }

    /**
     * 获取YYYY-MM-DD HH:mm:ss格式
     *
     * @return
     */
    public static String getTime() {
        return formatDate(new Date(), "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 获取YYYY-MM-DD HH:mm:ss.SSS格式
     *
     * @return
     */
    public static String getMsTime() {
        return formatDate(new Date(), "yyyy-MM-dd HHmmss.SSS");
    }

    /**
     * 获取YYYYMMDDHHmmss格式
     *
     * @return
     */
    public static String getAllTime() {
        return formatDate(new Date(), "yyyyMMddHHmmss");
    }

    /**
     * 获取YYYY-MM-DD HH:mm:ss格式
     *
     * @return
     */
    public static String getTime(Date date) {
        return formatDate(date, "yyyy-MM-dd HH:mm:ss");
    }

    public static String getChineseDay(Date date) {
        return formatDate(date, "yyyy年MM月dd日");
    }

    public static String formatDate(Date date, String pattern) {
        String formatDate = null;
        if (StringUtils.isNotBlank(pattern)) {
            formatDate = DateFormatUtils.format(date, pattern);
        } else {
            formatDate = DateFormatUtils.format(date, "yyyy-MM-dd");
        }
        return formatDate;
    }

    /**
     * @param s
     * @param e
     * @return boolean
     * @throws
     * @Title: compareDate
     * @Description:(日期比较，如果s>=e 返回true 否则返回false)
     * @author luguosui
     */
    public static boolean compareDate(String s, String e) {
        if (parseDate(s) == null || parseDate(e) == null) {
            return false;
        }
        return parseDate(s).getTime() >= parseDate(e).getTime();
    }

    /**
     * 格式化日期
     *
     * @return
     */
    public static Date parseDate(String date) {
        return parse(date, "yyyy-MM-dd");
    }

    /**
     * 格式化日期
     *
     * @return
     */
    public static Date parseTime(String date) {
        return parse(date, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 格式化日期
     *
     * @return
     */
    public static Date parse(String date, String pattern) {
        try {
            return DateUtils.parseDate(date, pattern);
        } catch (ParseException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 格式化日期
     *
     * @return
     */
    public static String format(Date date, String pattern) {
        return DateFormatUtils.format(date, pattern);
    }

    /**
     * 把日期转换为Timestamp
     *
     * @param date
     * @return
     */
    public static Timestamp format(Date date) {
        return new Timestamp(date.getTime());
    }

    /**
     * 校验日期是否合法
     *
     * @return
     */
    public static boolean isValidDate(String s) {
        return parse(s, "yyyy-MM-dd HH:mm:ss") != null;
    }

    /**
     * 校验日期是否合法
     *
     * @return
     */
    public static boolean isValidDate(String s, String pattern) {
        return parse(s, pattern) != null;
    }

    public static int getDiffYear(String startTime, String endTime) {
        DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        try {
            int years = (int) (((fmt.parse(endTime).getTime() - fmt.parse(
                    startTime).getTime()) / (1000 * 60 * 60 * 24)) / 365);
            return years;
        } catch (Exception e) {
            // 如果throw java.text.ParseException或者NullPointerException，就说明格式不对
            return 0;
        }
    }

    /**
     * <li>功能描述：时间相减得到天数
     *
     * @param beginDateStr
     * @param endDateStr
     * @return long
     * @author Administrator
     */
    public static long getDaySub(String beginDateStr, String endDateStr) {
        long day = 0;
        SimpleDateFormat format = new SimpleDateFormat(
                "yyyy-MM-dd");
        Date beginDate = null;
        Date endDate = null;

        try {
            beginDate = format.parse(beginDateStr);
            endDate = format.parse(endDateStr);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        day = (endDate.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);
        // System.out.println("相隔的天数="+day);

        return day;
    }

    /**
     * 得到n天之后的日期
     *
     * @param days
     * @return
     */
    public static String getAfterDayDate(String days) {
        int daysInt = Integer.parseInt(days);

        Calendar canlendar = Calendar.getInstance(); // java.util包
        canlendar.add(Calendar.DATE, daysInt); // 日期减 如果不够减会将月变动
        Date date = canlendar.getTime();

        SimpleDateFormat sdfd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateStr = sdfd.format(date);

        return dateStr;
    }

    /**
     * 得到n天之后是周几
     *
     * @param days
     * @return
     */
    public static String getAfterDayWeek(String days) {
        int daysInt = Integer.parseInt(days);

        Calendar canlendar = Calendar.getInstance(); // java.util包
        canlendar.add(Calendar.DATE, daysInt); // 日期减 如果不够减会将月变动
        Date date = canlendar.getTime();

        SimpleDateFormat sdf = new SimpleDateFormat("E");
        String dateStr = sdf.format(date);

        return dateStr;
    }

    /**
     * 更具开始时间分割月
     *
     * @param startTime
     * @param endTime
     * @return
     */
    public static Map<String, Map<String, Date>> getMonthStartAndEndTime(Date startTime, Date endTime) {
        Map<String, Map<String, Date>> stringMapHashMap = new LinkedHashMap<>();
        Long parseStart = parse(getMonth(startTime), "yyyy-MM").getTime();
        Long parseEnd = parse(getMonth(endTime), "yyyy-MM").getTime();
        Long time = endTime.getTime();
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(parseStart);
        int i = 1;
        while (time >= (parseStart = cal.getTimeInMillis())) {
            HashMap<String, Date> stringDateHashMap = new HashMap<>();
            if (i == 1) {
                stringDateHashMap.put("startTime", startTime);
            } else {
                stringDateHashMap.put("startTime", cal.getTime());
            }
            cal.add(Calendar.MONTH, 1);
            cal.add(Calendar.DATE, -1);
            if (parseEnd.equals(parseStart)) {
                stringDateHashMap.put("endTime", endTime);
            } else {
                stringDateHashMap.put("endTime", cal.getTime());
            }
            cal.add(Calendar.DATE, 1);
            stringMapHashMap.put(getMonth(new Date(parseStart)), stringDateHashMap);
            i++;
        }
        return stringMapHashMap;
    }

    /**
     * 更具开始时间分割周
     *
     * @return
     */
    public static Map<String, Map<String, Date>> getWeekStartAndEndTime(Date startTime, Date endTime) {
        Calendar cal = Calendar.getInstance();
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        cal.setTime(startTime);
        int start = cal.get(Calendar.WEEK_OF_YEAR);
        cal.setTime(endTime);
        int end = cal.get(Calendar.WEEK_OF_YEAR);
        Map<String, Map<String, Date>> stringMapHashMap = new LinkedHashMap<>();
        SimpleDateFormat simple = new SimpleDateFormat("yyyy-");
        for (int i = start; i <= end; i++) {
            cal.set(Calendar.WEEK_OF_YEAR, i);
            String format = simple.format(cal.getTime()) + (i <= 9 ? "0" + i : i);
            HashMap<String, Date> stringDateHashMap = new HashMap<>();
            if (start == i) {
                if (start == end) {
                    stringDateHashMap.put("startTime", startTime);
                    stringDateHashMap.put("endTime", endTime);
                } else {
                    stringDateHashMap.put("startTime", startTime);
                    cal.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
                    stringDateHashMap.put("endTime", cal.getTime());
                }
            } else if (end == i) {
                cal.set(Calendar.DAY_OF_WEEK, Calendar.MONTH);
                stringDateHashMap.put("startTime", cal.getTime());
                stringDateHashMap.put("endTime", endTime);
            } else {
                cal.set(Calendar.DAY_OF_WEEK, Calendar.MONTH);
                stringDateHashMap.put("startTime", cal.getTime());
                cal.set(Calendar.DAY_OF_WEEK, Calendar.SUNDAY);
                stringDateHashMap.put("endTime", cal.getTime());
            }
            stringMapHashMap.put(format, stringDateHashMap);
        }
        return stringMapHashMap;
    }

    /**
     * 更具开始时间分割日
     *
     * @return
     */
    public static Map<String, Map<String, Date>> getDayStartAndEndTime(Date startTime, Date endTime) {
        Map<String, Map<String, Date>> stringMapHashMap = new LinkedHashMap<>();
        Integer dayMun = getDayMun(startTime, endTime);
        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);
        for (int i = 0; i < dayMun; i++) {
            HashMap<String, Date> stringDateHashMap = new HashMap<>();
            if (i == 0) {
                stringDateHashMap.put("startTime", startTime);
            } else {
                cal.add(Calendar.SECOND, 1);
                stringDateHashMap.put("startTime", cal.getTime());
            }
            if (i == (dayMun - 1)) {
                stringDateHashMap.put("endTime", endTime);
            } else {
                cal.set(Calendar.HOUR_OF_DAY, 23);
                cal.set(Calendar.MINUTE, 59);
                cal.set(Calendar.SECOND, 59);
                stringDateHashMap.put("endTime", cal.getTime());
            }
            stringMapHashMap.put(getDay(cal.getTime()), stringDateHashMap);
        }
        return stringMapHashMap;
    }

    /**
     * 获取开始时间到结束时间的天数
     *
     * @param startTime
     * @param endTime
     * @return
     */
    public static Integer getDayMun(Date startTime, Date endTime) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            startTime = sdf.parse(sdf.format(startTime));
            endTime = sdf.parse(sdf.format(endTime));
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage(), e);
        }
        long days = (endTime.getTime() - startTime.getTime()) / (1000 * 3600 * 24);
        return (int) days + 1;
    }

    /**
     * 获取 yyyy-ww
     *
     * @param startTime
     * @return
     */
    public static String getWeek(Date startTime) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(startTime);
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        int weekYear = cal.get(Calendar.WEEK_OF_YEAR);
        return cal.get(Calendar.YEAR) + "-" + (weekYear <= 9 ? "0" + weekYear : weekYear);
    }

    public static List<String> getFiveYear() {
        List<String> list = new ArrayList<>();
        String year = getYear();
        list.add(year);
        Integer integer = Integer.valueOf(year);
        for (int i = 1; i < 6; i++) {
            list.add(String.valueOf(integer-i));
        }
        return list;
    }

    public static Map<String,String> startEndToDay() throws ParseException {
        Map<String,String> map = new HashMap<>();
        SimpleDateFormat sf=new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
        calendar.add(Calendar.DAY_OF_WEEK, -1);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        map.put("startDay", sf.format(calendar.getTime()));
        calendar.add(Calendar.DAY_OF_WEEK, 6);
        map.put("endDay", sf.format(calendar.getTime()));
        return map;
    }

    public static Map<String,Object> getLastWeekTime() {
        Map<String,Object> map = new HashMap<>();
        SimpleDateFormat sf=new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.setFirstDayOfWeek(Calendar.MONDAY);
        calendar.add(Calendar.WEEK_OF_YEAR, -1);
        calendar.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
        int i = calendar.get(Calendar.WEEK_OF_YEAR);
        map.put("week", i);
        map.put("weekTime", calendar.get(Calendar.YEAR)+"-"+(i<=9?"0"+i:i));
        map.put("startDay", calendar.get(Calendar.DAY_OF_MONTH));
        map.put("startMonth", calendar.get(Calendar.MONTH)+1);
        map.put("startTime", sf.format(calendar.getTime()));
        calendar.add(Calendar.DAY_OF_WEEK, 6);
        map.put("year", calendar.get(Calendar.YEAR));
        map.put("endDay", calendar.get(Calendar.DAY_OF_MONTH));
        map.put("endMonth", calendar.get(Calendar.MONTH)+1);
        map.put("endTime", sf.format(calendar.getTime()));
        return map;
    }

    public static Map<? extends String, ?> getLastMonthTime() throws ParseException {
        Map<String,Object> map = new HashMap<>();
        SimpleDateFormat sf=new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sfm=new SimpleDateFormat("yyyy-MM");
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.MONTH, -1);
        calendar.set(Calendar.DAY_OF_MONTH,1);
        map.put("startTime", sf.format(calendar.getTime()));
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        map.put("year", calendar.get(Calendar.YEAR));
        map.put("endDay", calendar.get(Calendar.DAY_OF_MONTH));
        map.put("endMonth", calendar.get(Calendar.MONTH)+1);
        map.put("endTime", sf.format(calendar.getTime()));
        map.put("monthTime", sfm.format(calendar.getTime()));
        calendar.add(Calendar.MONTH, -1);
        map.put("onMonthTime", sfm.format(calendar.getTime()));
        calendar.add(Calendar.MONTH, 1);
        calendar.add(Calendar.YEAR, -1);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        map.put("oldYear", calendar.get(Calendar.YEAR));
        map.put("oldMonth", calendar.get(Calendar.MONTH)+1);
        map.put("oldDay", calendar.get(Calendar.DAY_OF_MONTH));
        map.put("oldTime", sf.format(calendar.getTime()));
        map.put("oldMonthTime", sfm.format(calendar.getTime()));
        return map;
    }

}
