package com.qr.database.utils;

import com.aspose.words.Document;
import com.aspose.words.License;
import com.aspose.words.SaveFormat;
import com.qr.database.config.FreemarkerConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;

public class DocUtils {

    private static Logger log = LoggerFactory.getLogger(DocUtils.class);

    static {
        try {
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            InputStream license = new FileInputStream(loader.getResource("license.xml").getPath());
            License aposeLic = new License();
            aposeLic.setLicense(license);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    public static byte [] docToPdf(String fileInput) {
        Document doc = null;
        ByteArrayOutputStream byteArrayOutputStream = null;
        try {
            doc = new Document(fileInput);
            byteArrayOutputStream = new ByteArrayOutputStream();
            doc.save(byteArrayOutputStream, SaveFormat.PDF);
            return byteArrayOutputStream.toByteArray();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            if (byteArrayOutputStream !=null) {
                try {
                    byteArrayOutputStream.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
            if (doc != null) {
                try {
                    doc.deepClone();
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return new byte[]{};
    }
}
