package com.qr.database.utils;

import cn.afterturn.easypoi.entity.vo.TemplateExcelConstants;
import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.TemplateExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.afterturn.easypoi.word.WordExportUtil;
import com.qr.database.config.properties.GlobalProperties;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.ui.ModelMap;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class ExportExcelUtil {

    private static final Logger log = LoggerFactory.getLogger(ExportExcelUtil.class);

    public static String export(ModelMap modelMap, String templateFileName, String exportFileName, Map<String, Object> map) {
        TemplateExportParams params = new TemplateExportParams(gh(templateFileName), true);
        modelMap.put(TemplateExcelConstants.FILE_NAME, exportFileName);
        modelMap.put(TemplateExcelConstants.PARAMS, params);
        modelMap.put(TemplateExcelConstants.MAP_DATA, map);
        return TemplateExcelConstants.EASYPOI_TEMPLATE_EXCEL_VIEW;
    }

    private static String gh(String fileName) {
        return GlobalProperties.getFileUploadPath() + "export" + File.separator + fileName;
    }

    public static boolean export(Map<String, Object> map, String name) {
        Object fileName = map.get("fileName");
        if (fileName != null) {
            Workbook xwpfDocument = null;
            FileOutputStream fileOutputStream = null;
            try {
                TemplateExportParams params = new TemplateExportParams(gh(name), true);
                xwpfDocument = ExcelExportUtil.exportExcel(params, map);
                if (xwpfDocument != null) {
                    fileOutputStream = new FileOutputStream(GlobalProperties.getFileUploadPath() + fileName);
                    xwpfDocument.write(fileOutputStream);
                    return true;
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            } finally {
                if (fileOutputStream != null) {
                    try {
                        fileOutputStream.close();
                    } catch (IOException e) {
                        log.error(e.getMessage(), e);
                    }
                }
                if (xwpfDocument != null) {
                    try {
                        xwpfDocument.close();
                    } catch (IOException e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
        }
        return false;
    }

    public static boolean export(Map<String, Object> map, String name,OutputStream outputStream) {
        Workbook xwpfDocument = null;
        try {
            TemplateExportParams params = new TemplateExportParams(gh(name), true);
            xwpfDocument = ExcelExportUtil.exportExcel(params, map);
            if (xwpfDocument != null) {
                xwpfDocument.write(outputStream);
                return true;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            if (xwpfDocument != null) {
                try {
                    xwpfDocument.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return false;
    }

    public static boolean exportDocx(Map<String, Object> map, String name,OutputStream outputStream) {
        XWPFDocument xwpfDocument = null;
        try {
            System.out.println(map);
            xwpfDocument = WordExportUtil.exportWord07(GlobalProperties.getFileUploadPath() + "export" + File.separator + name, map);
            if (xwpfDocument != null) {
                xwpfDocument.write(outputStream);
                return true;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            if (xwpfDocument != null) {
                try {
                    xwpfDocument.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return false;
    }
}
