package com.qr.database.utils;

import com.qr.database.config.properties.GlobalProperties;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

public class FileUtil {

    private static Logger log = LoggerFactory.getLogger(FileUtil.class);

    /**
     * NIO way
     */
    public static byte[] toByteArray(String filename) {

        File f = new File(filename);
        if (!f.exists()) {
            log.error("文件未找到！" + filename);
            throw new RuntimeException("文件未找到！");
        }
        FileChannel channel = null;
        FileInputStream fs = null;
        try {
            fs = new FileInputStream(f);
            channel = fs.getChannel();
            ByteBuffer byteBuffer = ByteBuffer.allocate((int) channel.size());
            while ((channel.read(byteBuffer)) > 0) {
                // do nothing
                // System.out.println("reading");
            }
            return byteBuffer.array();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            try {
                channel.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            try {
                fs.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * 删除目录
     *
     * @author harry
     * @Date 2017/10/30 下午4:15
     */
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        return dir.delete();
    }

    public static boolean deleteFile(String filePath) {
        if (StringUtils.isNotBlank(filePath)) {
            return deleteDir(new File(GlobalProperties.getFileUploadPath()+filePath));
        }
        return false;
    }

    /**
     * 上传文件
     *
     * @param picture
     * @param fileSavePath
     */
    public static String upload(MultipartFile picture, String fileSavePath, String fileName) {
        File file = new File(fileSavePath);
        if (!file.exists()) {
            file.mkdirs();
        }
        File file1 = new File(file, fileName);
//        if (file1.exists()) {
//            return "";
//        }
        try {
            picture.transferTo(file1);
        } catch (IOException e) {
            log.error(e.getMessage(),e);
            return null;
        }
        return fileName;
    }
}