package com.qr.database.utils;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.ImageType;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.misc.BASE64Encoder;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class PdfUtils {

    private static Logger log = LoggerFactory.getLogger(PdfUtils.class);

    public static List<String> toImg(String pdfFilePath) {
        try {
            PDDocument doc = PDDocument.load(new File(pdfFilePath));
            return toImg(doc);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return new ArrayList<String>(0);
    }

    public static List<String> toImg(byte [] data) {
        try {
            PDDocument doc = PDDocument.load(data);
            return toImg(doc);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return new ArrayList<String>(1);
    }

    private static List<String> toImg(PDDocument doc) {
        List<String> strings = null;
        try {
            int pageCount = doc.getNumberOfPages();
            PDFRenderer pdfRenderer = new PDFRenderer(doc);
            BASE64Encoder encoder = new BASE64Encoder();
            strings = new ArrayList<>(pageCount);
            for (int pageIndex = 0; pageIndex < pageCount; pageIndex++) {
                BufferedImage image = pdfRenderer.renderImageWithDPI(pageIndex, 85, ImageType.RGB);
                ByteArrayOutputStream baos = null;
                try {
                    baos = new ByteArrayOutputStream();//io流
                    ImageIO.write(image, "png", baos);//写入流中
                    byte[] bytes = baos.toByteArray();//转换成字节
                    String png_base64 =  encoder.encodeBuffer(bytes).trim();//转换成base64串
                    png_base64 = png_base64.replaceAll("\n", "").replaceAll("\r", "");//删除 \r\n
                    strings.add(png_base64);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                } finally {
                    if (baos != null) {
                        baos.close();
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            if (doc != null) {
                try {
                    doc.close();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
        return strings;
    }

    public static void main(String[] args) throws InterruptedException {
        List<String> strings = toImg("C:\\Users\\hasee\\Documents\\WeChat Files\\li1210451792\\FileStorage\\File\\2019-07\\滴滴出行行程报销单_201907052050581.pdf");
        for (String s:strings) {
            System.out.println("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
            Thread.sleep(1000);
            System.out.println(s);
        }
        System.out.println(strings.size());
    }
}
