package com.qr.database.utils;

import com.qr.database.config.GlobalConstant;
import com.qr.database.config.properties.GlobalProperties;
import com.qr.database.model.Product;
import com.qr.database.model.Shenqian;
import com.qr.database.tips.ReturnResult;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class ShenQianPreviewUtil extends Thread {

    private String id;
    private Map<String, Object> shenqian;
    private Product product;
    private Integer docType;

    public ShenQianPreviewUtil(String id, Map<String, Object> shenqian, Product product, Integer docType) {
        super();
        this.id = id;
        this.shenqian = shenqian;
        this.product = product;
        this.docType = docType;
    }

    @Override
    public void run() {
        if (product==null) {
            RedisUtils.setIfPresent(id, new ReturnResult(400, "数据不存在"), 8, TimeUnit.MINUTES);
            return;
        }
        LinkedHashMap<String, Object> stringObjectHashMap = new LinkedHashMap<>();
        String surveyReport = product.getSurveyReport();
        if (StringUtils.isNotBlank(surveyReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——鉴定报告："+surveyReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "2"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(surveyReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + surveyReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + surveyReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "8"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", surveyReport);
            stringObjectHashMap1.put("docType", 1);
            if (docType==1) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "13"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String outgoingReport = product.getOutgoingReport();
        if (StringUtils.isNotBlank(outgoingReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——出厂报告："+outgoingReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "14"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(outgoingReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + outgoingReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + outgoingReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "20"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", outgoingReport);
            stringObjectHashMap1.put("docType", 2);
            if (docType==2) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "25"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String thirdpartySizeReport = product.getThirdpartySizeReport();
        if (StringUtils.isNotBlank(thirdpartySizeReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——第三方尺寸复验报告："+thirdpartySizeReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "26"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(thirdpartySizeReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + thirdpartySizeReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + thirdpartySizeReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "32"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", thirdpartySizeReport);
            stringObjectHashMap1.put("docType", 3);
            if (docType==3) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "37"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String thirdpartyPerReport = product.getThirdpartyPerReport();
        if (StringUtils.isNotBlank(thirdpartyPerReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——第三方性能报告："+thirdpartyPerReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "38"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(thirdpartyPerReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + thirdpartyPerReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + thirdpartyPerReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "43"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", thirdpartyPerReport);
            stringObjectHashMap1.put("docType", 4);
            if (docType==4) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "49"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String intoSizeReport = product.getIntoSizeReport();
        if (StringUtils.isNotBlank(intoSizeReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——入厂尺寸复验报告："+intoSizeReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "50"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(intoSizeReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + intoSizeReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + intoSizeReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "56"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", intoSizeReport);
            stringObjectHashMap1.put("docType", 5);
            if (docType==5) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "61"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String intoPerReport = product.getIntoPerReport();
        if (StringUtils.isNotBlank(intoPerReport)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——入厂性能复验报告："+intoPerReport.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "62"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(intoPerReport)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + intoPerReport));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + intoPerReport);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "68"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", intoPerReport);
            stringObjectHashMap1.put("docType", 6);
            if (docType==6) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "73"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String certification = product.getCertification();
        if (StringUtils.isNotBlank(certification)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——合格证："+certification.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "74"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(certification)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + certification));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + certification);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "80"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", certification);
            stringObjectHashMap1.put("docType", 7);
            if (docType==7) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "85"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        String subCertification = product.getSubCertification();
        if (StringUtils.isNotBlank(subCertification)) {
            HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
            stringObjectHashMap.put(product.getBatchNo()+"——子合格证："+subCertification.substring(11), stringObjectHashMap1);
            if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "86"), 8, TimeUnit.MINUTES)) {
                return ;
            }
            if (isPdf(subCertification)) {
                stringObjectHashMap1.put("img", PdfUtils.toImg(GlobalProperties.getFileUploadPath() + subCertification));
            } else {
                byte[] bytes = DocUtils.docToPdf(GlobalProperties.getFileUploadPath() + subCertification);
                if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "92"), 8, TimeUnit.MINUTES)) {
                    return ;
                }
                stringObjectHashMap1.put("img", PdfUtils.toImg(bytes));
            }
            stringObjectHashMap1.put("download", subCertification);
            stringObjectHashMap1.put("docType", 8);
            if (docType==8) {
                stringObjectHashMap1.put("activate", true);
            }
        }
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "97"), 8, TimeUnit.MINUTES)) {
            return ;
        }
        stringObjectHashMap.put("shenqian", shenqian);
        if (!RedisUtils.setIfPresent(id, new ReturnResult(300, "98"), 7, TimeUnit.MINUTES)) {
            return ;
        }
        RedisUtils.set(GlobalConstant.PREVIEW_FINISH+id, stringObjectHashMap, 5, TimeUnit.MINUTES);
        if (!RedisUtils.setIfPresent(id, new ReturnResult(200, "100", GlobalConstant.PREVIEW_FINISH+id), 4, TimeUnit.MINUTES)) {
            return ;
        }
    }

    public static boolean isPdf(String pdfFilePath) {
        if (pdfFilePath.lastIndexOf(".")<=0) {
            return false;
        }
        if ("pdf".equalsIgnoreCase(pdfFilePath.substring(pdfFilePath.lastIndexOf(".")+1))) {
            return true;
        }
        return false;
    }
}
