package com.qr.database.utils;

import com.qr.database.config.properties.GlobalProperties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @Description: ZipUtil
 * @Author: harry
 * @CreateDate: 2018/10/19 17:47
 * @UpdateUser:
 * @UpdateDate: 2018/10/19 17:47
 * @UpdateRemark:
 * @Version: 1.0
 */
public class ZipUtil {

    private static Logger log = LoggerFactory.getLogger(ZipUtil.class);

    public static ByteArrayInputStream toByteArrayInputStream(ByteArrayOutputStream out) {
        return new ByteArrayInputStream(out.toByteArray());
    }

    /**
     * 文件批量压缩
     *
     * @param parms
     */
    public static void batchFileToZIP(List<Map<String, Object>> parms, OutputStream OutPutStream) {
        ZipOutputStream zipOut = new ZipOutputStream(OutPutStream);
        InputStream inputStream = null;
        try {
            for (Map<String, Object> value : parms) {
                inputStream = (InputStream) value.get("file");
                // 使用指定名称创建新的 ZIP 条目 （通俗点就是文件名）
                ZipEntry zipEntry = new ZipEntry(String.valueOf(value.get("fileName")));
                // 开始写入新的 ZIP 文件条目并将流定位到条目数据的开始处
                zipOut.putNextEntry(zipEntry);
                byte[] b = new byte[1024];
                int length = 0;
                while ((length = inputStream.read(b)) != -1) {
                    zipOut.write(b, 0, length);
                }
                inputStream.close();
                zipEntry.clone();
                zipOut.closeEntry();
            }
        } catch (FileNotFoundException e) {
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
                zipOut.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    public static void batchDataToZIP(List<Map<String, Object>> parms, OutputStream OutPutStream) {
        ZipOutputStream zipOut = new ZipOutputStream(OutPutStream);
        try {
            for (Map<String, Object> value : parms) {
                byte[] inputStream = (byte[]) value.get("data");
                // 使用指定名称创建新的 ZIP 条目 （通俗点就是文件名）
                ZipEntry zipEntry = new ZipEntry(String.valueOf(value.get("fileName")));
                // 开始写入新的 ZIP 文件条目并将流定位到条目数据的开始处
                zipOut.putNextEntry(zipEntry);
                zipOut.write(inputStream);
                zipEntry.clone();
                zipOut.closeEntry();
            }
        } catch (FileNotFoundException e) {
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                zipOut.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    public static void batchAllToZIP(List<Map<String, Object>> parms, OutputStream OutPutStream) {
        ZipOutputStream zipOut = new ZipOutputStream(OutPutStream);
        InputStream input = null;
        try {
            for (Map<String, Object> value : parms) {
                Object data = value.get("data");
                ZipEntry zipEntry = null;
                try {
                    if (data != null) {
                            byte[] inputStream = (byte[]) data;
                            // 使用指定名称创建新的 ZIP 条目 （通俗点就是文件名）
                            zipEntry = new ZipEntry(String.valueOf(value.get("fileName")));
                            // 开始写入新的 ZIP 文件条目并将流定位到条目数据的开始处
                            zipOut.putNextEntry(zipEntry);
                            zipOut.write(inputStream);
                    } else if (value.get("file") != null) {
                        input = (InputStream) value.get("file");
                        // 使用指定名称创建新的 ZIP 条目 （通俗点就是文件名）
                        zipEntry = new ZipEntry(String.valueOf(value.get("fileName")));
                        // 开始写入新的 ZIP 文件条目并将流定位到条目数据的开始处
                        zipOut.putNextEntry(zipEntry);
                        byte[] b = new byte[1024];
                        int length = 0;
                        while ((length = input.read(b)) != -1) {
                            zipOut.write(b, 0, length);
                        }
                    }
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                } finally {
                    if (input != null) {
                        input.close();
                    }
                    if (zipEntry != null) {
                        zipEntry.clone();
                    }
                    if (zipOut != null) {
                        zipOut.closeEntry();
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }finally {
            try {
                if (input != null) {
                    input.close();
                }
                zipOut.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    public static Map<String, Object> getZipMap(String filePath) throws FileNotFoundException {
        HashMap<String, Object> stringObjectHashMap1 = new HashMap<>();
        File file = new File(GlobalProperties.getFileUploadPath() + filePath);
        stringObjectHashMap1.put("fileName", file.getName());
        stringObjectHashMap1.put("file", new FileInputStream(file));
        return stringObjectHashMap1;
    }

}
